# frozen_string_literal: true

require "faraday"
require "json"

module NHApi
  class Doujin
    attr_reader :id, :media_id, :title, :pages, :thumbnail, :cover, :upload_date, :tags, :num_pages, :num_favorites

    def initialize(attributes)
      @id = attributes["id"]
      @media_id = attributes["media_id"]
      @title = attributes["title"]
      @pages = build_pages(attributes["images"]["pages"])
      @thumbnail = NHApi::Thumbnail.new(attributes["images"]["thumbnail"].merge(media_id: media_id))
      @cover = NHApi::Cover.new(attributes["images"]["cover"].merge(media_id: media_id))
      @upload_date = attributes["upload_date"]
      @tags = attributes["tags"]
      @num_pages = attributes["num_pages"]
      @num_favorites = attributes["num_favorites"]
    end

    def self.find(id)
      response = Faraday.get("#{API_URL}/gallery/#{id}")
      attributes = JSON.parse(response.body)
      new(attributes)
    end

    def self.all(page = 1)
      response = Faraday.get("#{API_URL}galleries/all?page=#{page}")
      doujins = JSON.parse(response.body)["result"]
      doujins.map { |attributes| new(attributes) }
    end

    def related
      response = Faraday.get("#{API_URL}gallery/#{id}/related")
      doujins = JSON.parse(response.body)["result"]
      doujins.map { |attributes| NHApi::Doujin.new(attributes) }
    end

    private

    def build_pages(attributes)
      attributes.map.with_index do |page_data, page_index|
        NHApi::Page.new(page_data.merge(page_index: page_index + 1, media_id: media_id))
      end
    end
  end
end
